import QtQuick 2.0
import QtQuick.Window 2.0
import QtQuick.Controls 2.0
import MuseScore 3.0

MuseScore {
    menuPath: "Plugins.NoteNumber"
    description: "Note namber"
    version: "3.0"
    pluginType: "dialog"
    requiresScore: false
//Window {
    id: window
    width: 250
    height: 500
    property var keysModel: ["F","C","G","D","A","E","B","C♭","F♯","G♭","C♯","D♭","A♭","E♭","B♭","CHORD"]

    //property var keys: [0,1,0,1,0,0,1,0,1,0,1,0]
    property var noteName: ["F","C","G","D","A","E","B"]
    property var altSign: ["♭","♯"]
    property var listModel: []

    Component.onCompleted: { run(); }

    ListView {
        id: listView
        width: parent.width - 20
        height: parent.height - 70
        anchors.top: parent.top
        anchors.topMargin: 10
        anchors.horizontalCenter: parent.horizontalCenter
        header: Rectangle {
            id: headerItem
            width: listView.width
            height:30
            color: "#e0e0e0"
            border.width: 2
            Row {
                spacing: 5
                Text {
                    width: 50
                    height: 30
                    text: qsTr("key")
                    horizontalAlignment: Text.AlignHCenter
                    verticalAlignment: Text.AlignVCenter
                    font.weight: Font.Bold
                    font.capitalization: Font.AllUppercase;
                }
                Text {
                    width: listView.width - 55
                    height: 30
                    text: "measures"
                    verticalAlignment: Text.AlignVCenter
                    font.weight: Font.Bold
                    font.capitalization: Font.AllUppercase
                }
            }
        }

        delegate: Item {
            id: delegateItem
            width: listView.width
            height: 25
            Rectangle{
                anchors.fill: parent
                border.width: 1
                color: listView.currentIndex == index ? "#e0e0e0" : "white"
                MouseArea {
                    anchors.fill: parent
                    onClicked: listView.currentIndex = index
                }
                Row {
                    spacing: 5

                    Text {
                        width: 50
                        height: 25
                        text: keysModel[index]
                        horizontalAlignment: Text.AlignHCenter
                        verticalAlignment: Text.AlignVCenter
                    }
                    TextInput {
                        id: textInput
                        width: listView.width - 55
                        height: 25
                        //validator: RegularExpressionValidator {  regularExpression: /[0-9,-]+/ }//QtQuick 2.15
                        text: modelData
                        verticalAlignment: Text.AlignVCenter
                        onTextChanged: {
                            setModel(index, text);
                        }
                    }
                }
            }
        }
    }

    Button {
        text: qsTr("Start")
        anchors.right: parent.right
        anchors.bottom: parent.bottom
        anchors.rightMargin: 10
        anchors.bottomMargin: 10
        onClicked: start()
    }

    CheckBox {
        id: checkBox
        text: qsTr("only selected")
        anchors.left: parent.left
        anchors.bottom: parent.bottom
        anchors.leftMargin: 10
        anchors.bottomMargin: 10
    }

    //zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz

    function getHarmony() {
        var harmony = [];
        for(var i = 0; i < curScore.nmeasures; i++) { harmony[i] = ""; }
        for(i = 0; i < 16; i++) {
            var keysArr = listModel[i].split(",");
            for(var ii = 0; ii < keysArr.length; ii++) {
                var keyArr = keysArr[ii].split("-");
                if(keyArr.length > 1) {
                    var bottomNumber = Number(keyArr[0]);
                    var topNumber = Number(keyArr[1]);
                    while(bottomNumber <= topNumber) {
                        harmony[bottomNumber - 1] = keysModel[i];
                        bottomNumber++;
                    }
                } else {
                    var numberKey = Number(keysArr[ii]);
                    harmony[numberKey - 1] = keysModel[i];
                }
            }
        }
        return harmony;
    }

    function run() {
        listModel = [];
        for(var i = 0; i < 16; i++) {
            listModel[i] = "";
        }
        listView.model = listModel;
    }

    function setModel(id, text) {
        listModel[id] = text;
    }

    //zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz

    function getTpcIndex(tpc) {
        var alteration = -2;
        var tpcIndex = tpc;
        while (tpcIndex > 5) {
            tpcIndex -= 7;
            alteration++;
        }
        tpcIndex++;
        return [tpcIndex, alteration];
    }

    function getNoteName(tpc) {
        var tpcIndex = getTpcIndex(tpc);
        var name = noteName[tpcIndex[0]];
        for (var alteration = tpcIndex[1]; alteration !== 0; alteration < 0 ? alteration++ : alteration--) {
            name += altSign[alteration < 0 ? 0 : 1];
        }
        return name;
    }

    function getStage(notesArr) {
        var result = notesArr[1].charCodeAt(0) - notesArr[0].charCodeAt(0);
        if(result < 0) { result += 7; }
        return result + 1;
    }

    function getNoteText(notes, harmonyMeasure) {
        var noteText = "";
        for (var i = 0; i < notes.length; i++) {
            var note = notes[i];
            if ( i > 0 )
                noteText  = "\n" + noteText;
            if (typeof note.pitch === "undefined")
                continue

            var stage = "" + getStage([harmonyMeasure, getNoteName(note.tpc)]);
            noteText  = stage + noteText;
        }

        var textHord = newElement(Element.STAFF_TEXT);
        textHord.text = noteText;
        return textHord;
    }

    function setNoteText() {
        var harmony = getHarmony();
        var harmonyMeasure = keysModel[listView.currentIndex];
        var numMeasure = 0;
        var noteText;
        var cursor = curScore.newCursor();
        for (var staff = 0; staff < curScore.nstaves; staff++) {
            for (var voice = 0; voice < 4; voice++) {
                cursor.rewind(0);
                cursor.voice = voice;
                cursor.staffIdx = staff;
                while (cursor.element) {
                    if(checkBox.checkState === Qt.Unchecked) {
                      if(cursor.measure.firstSegment.tick === cursor.segment.tick){
                        harmonyMeasure = harmony[numMeasure];
                        numMeasure++;
                      }
                    }
                    if ((harmonyMeasure != "") && (cursor.element.type === Element.CHORD)) {
                        for (var i = 0; i < cursor.element.graceNotes.length; i++) {
                            noteText = getNoteText(cursor.element.graceNotes[i].notes, harmonyMeasure)
                            cursor.add(noteText);
                        }
                        noteText = getNoteText(cursor.element.notes, harmonyMeasure)
                        cursor.add(noteText);
                    }
                    cursor.next();
                }
            }
        }
    }

    //zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz

    function log(text, value){
      console.log(text + ", value: " + value + ", type: " + typeof(value))
    }

    function start() {
        curScore.startCmd();
        setNoteText();
        curScore.endCmd();
        Qt.quit();
    }
}
